/**
 * 
 * AiCiA - Android IRC Client
 * 
 * Copyright (C)2010 GORRY.
 * 
 */

package net.gorry.aicia;

import java.util.ArrayList;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.media.AudioManager;
import android.media.ToneGenerator;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.os.RemoteException;
import android.os.SystemClock;
import android.os.Vibrator;
import android.text.SpannableStringBuilder;
import android.util.Log;

/**
 * 
 * IRCサービス
 * 
 * @author gorry
 *
 */
public class IRCService extends Service   {
	private static final String TAG = "IRCService";
	private static final boolean V = false;
	private static final boolean D = false;

	/** IRCServiceのアクション名 */
	public static final String ACTION = "IRCService";

	/** IRCServiceの機能番号 */
	public static final int RECEIVE_MESSAGE = 1;
	/** */
	public static final int CREATE_SERVER = 2;
	/** */
	public static final int REMOVE_SERVER = 3;
	/** */
	public static final int CREATE_CHANNEL = 4;
	/** */
	public static final int REMOVE_CHANNEL = 5;
	/** */
	public static final int CHANGE_NICK = 6;
	/** */
	public static final int CHANGE_TOPIC = 7;
	/** */
	public static final int PING_PONG = 8;

	private static final int NOTIFY_SYSTEM = 0;
	private static final int NOTIFY_LOWMEMORY = 1;
	private static final int NOTIFY_SERVER_MESSAGE = 10000;

	private IRCServerList ircServerList;
	private Context me;

	private NotificationManager nm;
	private String mInputBox = "";
	private int mInputBoxSelStart = 0;
	private int mInputBoxSelEnd = 0;
	private boolean mDonate = false;



	/*
	 * 作成
	 * @see android.app.Service#onCreate()
	 */
	@Override
	public void onCreate() {
		if (V) Log.v(TAG, "onCreate()");
		nm = (NotificationManager)getSystemService(Context.NOTIFICATION_SERVICE);
	}

	/*
	 * 開始
	 * @see android.app.Service#onStart(android.content.Intent, int)
	 */
	@Override
	public void onStart(final Intent intent, final int startId) {
		if (V) Log.v(TAG, "onStart()");
		final Bundle extras = intent.getExtras();
		if (extras != null) {
			mDonate = (extras.getBoolean("donate"));
		}
		SharedPreferences pref = getApplicationContext().getSharedPreferences("ircservice", 0);
		final boolean killedByLowMemory = pref.getBoolean("killedbylowmemory", false);
		if (killedByLowMemory) {
			showNotification("Restart", (mDonate ? "AiCiA (DONATED)" : "AiCiA"), getString(R.string.ircservice_java_restartircservicebylowmemory), NOTIFY_SYSTEM);
			final SharedPreferences.Editor editor = pref.edit();
			editor.putBoolean("killedbylowmemory", false);
			editor.commit();
		} else {
			showNotification("Start", (mDonate ? "AiCiA (DONATED)" : "AiCiA"), getString(R.string.ircservice_java_startircservice), NOTIFY_SYSTEM);
		}
		clearNotification(NOTIFY_LOWMEMORY);
		pref = null;
	}

	/*
	 * 破棄
	 * @see android.app.Service#onDestroy()
	 */
	@Override
	public void onDestroy() {
		if (V) Log.v(TAG, "onDestroy()");
		stopTimer();
		showNotification("End", (mDonate ? "AiCiA (DONATED)" : "AiCiA"), getString(R.string.ircservice_java_shutdownircservice), NOTIFY_SYSTEM);
		SharedPreferences pref = getApplicationContext().getSharedPreferences("ircservice", 0);
		final SharedPreferences.Editor editor = pref.edit();
		editor.putBoolean("killedbylowmemory", false);
		editor.commit();
		pref = null;
		clearNotification(NOTIFY_SYSTEM);
		clearNotification(NOTIFY_LOWMEMORY);
	}

	// バインド
	/*
	 * バインド
	 * @see android.app.Service#onBind(android.content.Intent)
	 */
	@Override
	public IBinder onBind(final Intent intent) {
		if (V) Log.v(TAG, "onBind()");
		me = this;
		final Bundle extras = intent.getExtras();
		if (extras != null) {
			mDonate = (extras.getBoolean("donate"));
		}

		SystemConfig.setContext(me);
		SystemConfig.loadConfig();

		if (ircServerList == null) {
			ircServerList = new IRCServerList(me, true);
			ircServerList.addEventListener(new IRCServerListListener());
			ircServerList.reloadList();
		}
		if (!IIRCService.class.getName().equals(intent.getAction())) {
			return null;
		}

		startTimer();
		return ircServiceIf;
	}

	/*
	 * 再バインド
	 * @see android.app.Service#onRebind(android.content.Intent)
	 */
	@Override
	public void onRebind(final Intent intent) {
		if (V) Log.v(TAG, "onRebind()");
	}

	/*
	 * バインド解除
	 * @see android.app.Service#onUnbind(android.content.Intent)
	 */
	@Override
	public boolean onUnbind(final Intent intent) {
		if (V) Log.v(TAG, "onUnbind()");
		return true;
	}

	@Override
	public void onLowMemory() {
		if (V) Log.v(TAG, "onLowMemory()");
		showNotification("Low Memory", (mDonate ? "AiCiA (DONATED)" : "AiCiA"), getString(R.string.ircservice_java_onlowmemory), NOTIFY_LOWMEMORY);
		SharedPreferences pref = getApplicationContext().getSharedPreferences("ircservice", 0);
		final SharedPreferences.Editor editor = pref.edit();
		editor.putBoolean("killedbylowmemory", true);
		editor.commit();
		pref = null;
		return;
	}

	//
	/**
	 * サーバリストイベントリスナ
	 */
	private class IRCServerListListener implements IRCServerListEventListener {
		public void receiveMessageToChannel(final String serverName, final String channel, final String nick, final String dateMsg, final SpannableStringBuilder ssb, final SpannableStringBuilder ssbOther, final boolean alert) {
			if (D) Log.d(TAG, "receiveMessageToChannel()");
			if (alert) {
				doAlert();
			}
			final Intent intent = new Intent(ACTION);
			intent.putExtra("msg", IRCService.RECEIVE_MESSAGE);
			intent.putExtra("server", serverName);
			intent.putExtra("channel", channel);
			intent.putExtra("nick", nick);
			intent.putExtra("date", dateMsg);
			intent.putExtra("ssb", ssb);
			intent.putExtra("ssbOther", ssbOther);
			intent.putExtra("alert", alert);
			sendBroadcast(intent);
			if (SystemConfig.checkConnectWaitSec > 0) {
				ircServerList.setNextKeepAliveTime(serverName, getTimerCount()+SystemConfig.checkConnectWaitSec);
			}
		}

		public void createServer(final String serverName, final String dateMsg) {
			if (D) Log.d(TAG, "createServer()");
			ircServerList.setCurrentServerName(serverName);
			final Intent intent = new Intent(ACTION);
			intent.putExtra("msg", IRCService.CREATE_SERVER);
			intent.putExtra("server", serverName);
			intent.putExtra("date", dateMsg);
			sendBroadcast(intent);
		}

		public void removeServer(final String serverName, final String dateMsg) {
			if (D) Log.d(TAG, "removeServer()");
			final Intent intent = new Intent(ACTION);
			intent.putExtra("msg", IRCService.REMOVE_SERVER);
			intent.putExtra("server", serverName);
			intent.putExtra("date", dateMsg);
			sendBroadcast(intent);
		}

		public void createChannel(final String serverName, final String channel, final String dateMsg) {
			if (D) Log.d(TAG, "createChannel()");
			ircServerList.setCurrentServerName(serverName);
			ircServerList.setCurrentChannel(serverName, channel);
			final Intent intent = new Intent(ACTION);
			intent.putExtra("msg", IRCService.CREATE_CHANNEL);
			intent.putExtra("server", serverName);
			intent.putExtra("channel", channel);
			intent.putExtra("date", dateMsg);
			sendBroadcast(intent);
		}

		public void removeChannel(final String serverName, final String channel, final String dateMsg) {
			if (D) Log.d(TAG, "removeChannel()");
			final Intent intent = new Intent(ACTION);
			intent.putExtra("msg", IRCService.REMOVE_CHANNEL);
			intent.putExtra("server", serverName);
			intent.putExtra("channel", channel);
			intent.putExtra("date", dateMsg);
			sendBroadcast(intent);
		}

		public void changeNick(final String serverName, final String oldNick, final String newNick, final String dateMsg) {
			if (D) Log.d(TAG, "changeNick()");
			final Intent intent = new Intent(ACTION);
			intent.putExtra("msg", IRCService.CHANGE_NICK);
			intent.putExtra("server", serverName);
			intent.putExtra("oldnick", oldNick);
			intent.putExtra("newnick", newNick);
			intent.putExtra("date", dateMsg);
			sendBroadcast(intent);
		}

		public void changeTopic(final String serverName, final String channel, final String topic, final String dateMsg) {
			if (D) Log.d(TAG, "changeTopic()");
			final Intent intent = new Intent(ACTION);
			intent.putExtra("msg", IRCService.CHANGE_TOPIC);
			intent.putExtra("server", serverName);
			intent.putExtra("channel", channel);
			intent.putExtra("topic", topic);
			intent.putExtra("date", dateMsg);
			sendBroadcast(intent);
		}

		public void receiveConnect(final String serverName) {
			if (D) Log.d(TAG, "receiveConnect()");
			if (D) ircServerList.echoMessageToChannel(serverName, IRCMsg.sSystemChannelName, "receiveConnect()");
			showNotification("Connect " + serverName, (mDonate ? "AiCiA (DONATED)" : "AiCiA"),
					String.format(me.getString(R.string.ircservice_java_connectserver),serverName),
					NOTIFY_SERVER_MESSAGE + ircServerList.getServerId(serverName)
			);
			ircServerList.setNextReconnectTime(serverName, 0);
			if (SystemConfig.checkConnectWaitSec > 0) {
				ircServerList.setNextKeepAliveTime(serverName, getTimerCount()+SystemConfig.checkConnectWaitSec);
			}
		}

		public void receiveDisconnect(final String serverName) {
			if (D) Log.d(TAG, "receiveDisconnect()");
			if (D) ircServerList.echoMessageToChannel(serverName, IRCMsg.sSystemChannelName, "receiveDisconnect()");
			showNotification("Disconnect " + serverName, (mDonate ? "AiCiA (DONATED)" : "AiCiA"),
					String.format(me.getString(R.string.ircservice_java_disconnectserver),serverName),
					NOTIFY_SERVER_MESSAGE + ircServerList.getServerId(serverName)
			);
			if (ircServerList.getNeedReconnect(serverName)) {
				if (SystemConfig.autoReconnectWaitSec > 0) {
					ircServerList.setNextReconnectTime(serverName, getTimerCount()+SystemConfig.autoReconnectWaitSec);
					ircServerList.echoMessageToChannel(serverName, IRCMsg.sSystemChannelName, "Wait " + SystemConfig.autoReconnectWaitSec + " seconds for retry.");
				}
			}
			ircServerList.setNextKeepAliveTime(serverName, 0);
		}
	}

	/**
	 * タイマー処理
	 */
	private boolean mTimerStop = false;
	private static final long mTimerStep = 1000;
	private long mTimerCount = 0;
	private long mTimerNext = 0;
	private static final int TIMER = 1;
	private final Handler mTimerHandler = new Handler() {
		@Override
		public void handleMessage(final Message msg) {
			if (msg.what == TIMER) {
				while (SystemClock.uptimeMillis() > mTimerNext) {
					mTimerNext += mTimerStep;
					mTimerCount++;
					onTimer(mTimerCount);
				}
				if (mTimerStop == false) {
					final Message msg2 = mTimerHandler.obtainMessage(TIMER);
					sendMessageAtTime(msg2, mTimerNext);
				}
			}
		}
	};
	private void startTimer() {
		if (V) Log.v(TAG, "startTimer()");
		mTimerStop = false;
		mTimerNext = SystemClock.uptimeMillis();
		final Message msg = mTimerHandler.obtainMessage(TIMER);
		mTimerHandler.sendMessageAtTime(msg, mTimerNext+mTimerStep);
	}
	private void stopTimer() {
		if (V) Log.v(TAG, "stopTimer()");
		mTimerStop = true;
	}
	private long getTimerCount() {
		return mTimerCount;
	}
	private void onTimer(final long timerCount) {
		final String [] servers = ircServerList.getServerList();
		for (int i=0; i<servers.length; i++) {
			final IRCServer ircServer = ircServerList.getServer(servers[i]);

			// KeepAlive送出＆切断検出
			final long t1 = ircServer.getNextKeepAliveTime();
			if ((t1 > 0) && (t1 < timerCount)) {
				if (SystemConfig.checkConnectWaitSec > 0) {
					if (D) Log.d(TAG, "ontimer(): try keepalive");
					if (SystemConfig.verbose >= 1) ircServer.echoChannel(IRCMsg.sSystemChannelName, "Post KeepAlive packet", true);
					ircServer.setNextKeepAliveTime(timerCount+SystemConfig.checkConnectWaitSec);
					ircServer.sendKeepAliveMessage();
					if (ircServer.isDisconnected()) {
						ircServer.setNextKeepAliveTime(0);
						if (D) Log.d(TAG, "ontimer(): lost connection");
						ircServer.echoChannel(IRCMsg.sSystemChannelName, "Lost connection", true);
						ircServer.disconnect();
						ircServer.connect();
					}
				} else {
					ircServer.setNextKeepAliveTime(0);
				}
			}

			// 自動再接続
			final long t2 = ircServer.getNextReconnectTime();
			if ((t2 > 0) && (t2 < timerCount)) {
				if (D) Log.d(TAG, "ontimer(): try reconnect");
				ircServer.echoChannel(IRCMsg.sSystemChannelName, "Try reconnect", true);
				ircServer.connect();
			}
		}

		/*
		// ActivityMainにPingを打つ
		final Intent intent = new Intent(ACTION);
		intent.putExtra("msg", IRCService.PING_PONG);
		sendBroadcast(intent);
		 */
	}

	/**
	 * 通知表示
	 * @param ticker ティッカー
	 * @param title タイトル
	 * @param message メッセージ
	 * @param id ID
	 */
	private void showNotification(final String ticker, final String title, final String message, final int id) {
		if (V) Log.v(TAG, "showNotification()");
		int icon = R.drawable.icon;
		final Intent intent = new Intent(this, net.gorry.aicia.ActivityMain.class);
		if (id == 1) {
			icon = R.drawable.icon_warn;
		} else if (id >= NOTIFY_SERVER_MESSAGE) {
			final int serverId = id - NOTIFY_SERVER_MESSAGE;
			if (serverId < ircServerList.getNumServers()) {
				intent.putExtra("serverName", ircServerList.getServerName(serverId));
				intent.setData((Uri.parse("foobar://"+SystemClock.elapsedRealtime())));
			}
			icon = R.drawable.icon_server;
		}
		final Notification notification = new Notification(icon, ticker, System.currentTimeMillis());
		final PendingIntent pi = PendingIntent.getActivity(this, 0, intent, 0);
		notification.setLatestEventInfo(this, title, message, pi);
		nm.cancel(id);
		nm.notify(id, notification);
	}

	/**
	 * 通知表示の消去
	 * @param ticker ティッカー
	 * @param title タイトル
	 * @param message メッセージ
	 * @param id ID
	 */
	private void clearNotification(final int id) {
		if (V) Log.v(TAG, "clearNotification()");
		nm.cancel(id);
	}

	/**
	 * フラット化したサーバ名・チャンネルリストの取得
	 * @param flatServerName
	 * @param flatChannel
	 * @param flatChannelUpdated
	 * @return
	 */
	private int getFlatServerChannelList(final ArrayList<String> flatServerName, final ArrayList<String> flatChannel, final ArrayList<Boolean> flatChannelUpdated ) {
		int noFlat = 0;
		int select = -1;
		final String currentServerName = ircServerList.getCurrentServerName();
		if (currentServerName == null) return -1;
		final String currentChannel = ircServerList.getCurrentChannel(currentServerName);
		if (currentChannel == null) return -1;
		final String[] svList = ircServerList.getServerList();
		for (int i=0; i<svList.length; i++) {
			final String[] chList = ircServerList.getChannelList(svList[i]);
			final Boolean[] chuList = ircServerList.getChannelUpdatedList(svList[i]);
			for (int j=0; j<chList.length; j++) {
				flatServerName.add(svList[i]);
				flatChannel.add(chList[j]);
				flatChannelUpdated.add(chuList[j]);
				if (svList[i].equalsIgnoreCase(currentServerName)
						&& chList[j].equalsIgnoreCase(currentChannel)) {
					select = noFlat;
				}
				noFlat++;
			}
		}
		return select;
	}

	/**
	 * IRCサービスAPI
	 */
	private final IIRCService.Stub ircServiceIf = new IIRCService.Stub() {
		public void shutdown() {
			if (V) Log.v(TAG, "shutdown()");
			ircServerList.disconnectAll();
			stopSelf();
		}

		public void reloadSystemConfig() {
			if (V) Log.v(TAG, "reloadSystemConfig()");
			SystemConfig.loadConfig();
		}

		public CharSequence getSpanChannelLog(final String serverName, final String channel) throws RemoteException {
			if (V) Log.v(TAG, "getChannelLog()");
			return ircServerList.getSpanChannelLog(serverName, channel);
		}

		public String getNick(final String serverName) throws RemoteException {
			if (V) Log.v(TAG, "getNick()");
			return ircServerList.getNick(serverName);
		}

		public String getTopic(final String serverName, final String channel) throws RemoteException {
			if (V) Log.v(TAG, "getTopic()");
			return ircServerList.getTopic(serverName, channel);
		}

		public void sendCommandLine(final String serverName, final String channel, final String message) throws RemoteException {
			if (V) Log.v(TAG, "sendCommandLine()");
			ircServerList.sendCommandLine(serverName, channel, message);
		}

		public void sendMessageToChannel(final String serverName, final String channel, final String message) throws RemoteException {
			if (V) Log.v(TAG, "sendMessageToChannel()");
			ircServerList.sendMessageToChannel(serverName, channel, message);
			ircServerList.setNextKeepAliveTime(serverName, getTimerCount()+SystemConfig.checkConnectWaitSec);
		}

		public void echoMessageToChannel(final String serverName, final String channel, final String message) throws RemoteException {
			if (V) Log.v(TAG, "echoMessageToChannel()");
			ircServerList.echoMessageToChannel(serverName, channel, message);
		}

		public void connectServer(final String serverName) throws RemoteException {
			if (V) Log.v(TAG, "connectServer()");
			ircServerList.connectServer(serverName);
		}

		public void disconnectServer(final String serverName) throws RemoteException {
			if (V) Log.v(TAG, "disconnectServer()");
			ircServerList.disconnectServer(serverName);
		}

		public void connectAuto() throws RemoteException {
			if (V) Log.v(TAG, "connectAuto()");
			ircServerList.connectAuto();
		}

		public void disconnectAll() throws RemoteException {
			if (V) Log.v(TAG, "disconnectAll()");
			ircServerList.disconnectAll();
		}

		public void addNewServer() throws RemoteException {
			if (V) Log.v(TAG, "addNewServer()");
			final int num = ircServerList.getNumServers();
			IRCServerConfig config = new IRCServerConfig(me);
			config.loadConfig(num);
			ircServerList.addServer(config);
			config = null;
		}

		public void reloadServerConfig(final int serverId) throws RemoteException {
			if (V) Log.v(TAG, "reloadServerConfig()");
			ircServerList.reloadServerConfig(serverId);
			final String serverName = ircServerList.getServerName(serverId);
			showNotification("Update " + serverName, (mDonate ? "AiCiA (DONATED)" : "AiCiA"),
					String.format(me.getString(R.string.ircservice_java_updateserver),serverName),
					NOTIFY_SERVER_MESSAGE + ircServerList.getServerId(serverName)
			);
		}

		public void removeServer(final int serverId) throws RemoteException {
			if (V) Log.v(TAG, "removeServer()");
			ircServerList.removeServer(serverId);
		}

		public void setCurrentServerName(final String serverName) throws RemoteException {
			if (V) Log.v(TAG, "setCurrentServerName()");
			ircServerList.setCurrentServerName(serverName);
		}

		public String getCurrentServerName() throws RemoteException {
			if (V) Log.v(TAG, "getCurrentServerName()");
			return ircServerList.getCurrentServerName();
		}

		public void setCurrentChannel(final String serverName, final String channel) throws RemoteException {
			if (V) Log.v(TAG, "setCurrentChannel()");
			ircServerList.setCurrentChannel(serverName, channel);
		}

		public String getCurrentChannel(final String serverName) throws RemoteException {
			if (V) Log.v(TAG, "getCurrentChannel()");
			return ircServerList.getCurrentChannel(serverName);
		}

		public String[] getServerList() throws RemoteException {
			if (V) Log.v(TAG, "getServerList()");
			return ircServerList.getServerList();
		}

		public String[] getChannelList(final String serverName) throws RemoteException {
			if (V) Log.v(TAG, "getChannelList()");
			return ircServerList.getChannelList(serverName);
		}

		public boolean[] getChannelUpdatedList(final String serverName) throws RemoteException {
			if (V) Log.v(TAG, "getChannelUpdatedList()");
			final Boolean[] b1 = ircServerList.getChannelUpdatedList(serverName);
			final int num = b1.length;
			final boolean[] b2 = new boolean[num];
			for (int i=0; i<num; i++) {
				b2[i] = b1[i];
			}
			return b2;
		}

		public boolean getChannelUpdated(final String serverName, final String channel) throws RemoteException {
			if (V) Log.v(TAG, "getChannelUpdated()");
			return ircServerList.getChannelUpdated(serverName, channel);
		}

		public void clearChannelUpdated(final String serverName, final String channel) throws RemoteException {
			if (V) Log.v(TAG, "clearChannelUpdated()");
			ircServerList.clearChannelUpdated(serverName, channel);
		}

		public void clearChannelUpdatedAll() throws RemoteException {
			if (V) Log.v(TAG, "clearChannelUpdatedAll()");
			for (int i=0; i<ircServerList.getNumServers(); i++) {
				final IRCServer ircServer = ircServerList.getServer(i);
				for (int j=0; j<ircServer.getNumChannels(); j++) {
					ircServer.clearChannelUpdated(j);
				}
			}
		}

		public String[] getUserList(final String serverName, final String channel) throws RemoteException {
			if (V) Log.v(TAG, "getChannelList()");
			return ircServerList.getUserList(serverName, channel);
		}

		public boolean changeNextChannel(final int dir, final int mode) throws RemoteException {
			if (V) Log.v(TAG, "changeNextChannel()");
			final ArrayList<String> flatServerName = new ArrayList<String>();
			final ArrayList<String> flatChannel = new ArrayList<String>();
			final ArrayList<Boolean> flatChannelUpdated = new ArrayList<Boolean>();
			final int currentId = getFlatServerChannelList(flatServerName, flatChannel, flatChannelUpdated);
			final int numFlat = flatServerName.size();
			if (numFlat < 1) return false;
			int newId = currentId;
			while (true) {
				if (dir > 0) {
					newId++;
					if (newId >= numFlat) newId = 0;
				} else {
					newId--;
					if (newId < 0) newId = numFlat-1;
				}
				if (newId == currentId) break;
				if (flatChannel.get(newId).equals(IRCMsg.sSystemChannelName)) continue;
				if (mode == 0) break;
				if (flatChannelUpdated.get(newId)) break;
			}
			if (newId == currentId) return false;
			ircServerList.setCurrentServerName(flatServerName.get(newId));
			ircServerList.setCurrentChannel(flatServerName.get(newId), flatChannel.get(newId));
			return true;
		}

		public void joinChannel(final String serverName, final String channel) throws RemoteException {
			if (V) Log.v(TAG, "joinChannel()");
			ircServerList.joinChannel(serverName, channel);
		}

		public void joinChannels(final String serverName, final String channels) throws RemoteException {
			if (V) Log.v(TAG, "joinChannels()");
			ircServerList.joinChannels(serverName, channels);
		}

		public void partChannel(final String serverName, final String channel) throws RemoteException {
			if (V) Log.v(TAG, "partChannel()");
			ircServerList.joinChannel(serverName, channel);
		}

		public void saveInputBox(final String input, final int selStart, final int selEnd) throws RemoteException {
			if (V) Log.v(TAG, "saveInputBox()");
			mInputBox = new String(input);
			mInputBoxSelStart = selStart;
			mInputBoxSelEnd = selEnd;
		}

		public String loadInputBox() throws RemoteException {
			if (V) Log.v(TAG, "loadInputBox()");
			return new String(mInputBox);
		}

		public int loadInputBoxSelStart() throws RemoteException {
			if (V) Log.v(TAG, "loadInputBoxSelStart()");
			return mInputBoxSelStart;
		}

		public int loadInputBoxSelEnd() throws RemoteException {
			if (V) Log.v(TAG, "loadInputBoxSelEnd()");
			return mInputBoxSelEnd;
		}

		public void receivePong() throws RemoteException {
			if (V) Log.v(TAG, "receivePong()");
		}

		public boolean isTIGMode(final String serverName) throws RemoteException {
			if (V) Log.v(TAG, "isTIGMode()");
			return ircServerList.isTIGMode(serverName);
		}

	};

	/**
	 * アラート処理
	 */
	private void doAlert() {
		if (SystemConfig.vibrateOnAlert) {
			new Thread(new Runnable() {	public void run() {
				final Vibrator vib = (Vibrator)getSystemService(VIBRATOR_SERVICE);
				final long[] pat = {200,200,200,200,200,200,200,200};
				vib.vibrate(pat, -1);
			} }).start();
		}
		if (SystemConfig.ringOnAlert) {
			new Thread(new Runnable() {	public void run() {
				final ToneGenerator tg = new ToneGenerator(
						AudioManager.STREAM_SYSTEM,
						ToneGenerator.MAX_VOLUME);
				tg.startTone(ToneGenerator.TONE_PROP_PROMPT);
				try {
					Thread.sleep(500);
				} catch (final InterruptedException e) {
					//
				}
				tg.stopTone();
			} }).start();
		}
	}

}
